superstrict
import "shader_lib.bmx"
import "vector3.bmx"
global bumpMapShader:TOpenGLShader 'Shader-Objekt
global colorMapUniform:TTextureUniform 'Uniform-Objekte
global alphaMapUniform:TTextureUniform
global bumpMapUniform:TTextureUniform
global lightDirUniform:TUniform
global normalVectorAttribute:TVertexAttribute 'VertexAttribute-Objekte
global localXAttribute:TVertexAttribute
global localYAttribute:TVertexAttribute

function LoadBumpMapShader(vertexUrl:object, fragmentUrl:object) 'Laedt den BumpMap-Shader bestehend
		'aus Vertex- und Fragmentshader und initialisiert obige Objekte

	bumpMapShader = new TOpenGLShader
	if not bumpMapShader.Init(vertexUrl, fragmentUrl) then runtimeerror "Error loading bumpmap shader"
	colorMapUniform = bumpMapShader.getTextureUniform("color")
	alphaMapUniform = bumpMapShader.getTextureUniform("alpha")
	bumpMapUniform = bumpMapShader.getTextureUniform("bump")
	lightDirUniform = bumpMapShader.getUniform("lightDir")
	normalVectorAttribute = bumpMapShader.getAttribute("normalVector")
	localXAttribute = bumpMapShader.getAttribute("localX")
	localYAttribute = bumpMapShader.getAttribute("localY")
	CheckError

end function
'Eine BumpMap-Sprite ist ein Containertyp, der drei Texturen beinhaltet
'Diese koennen dann mit einem Funktionsaufruf an die drei Uniform-Variablen
'{color, alpha, bump}MapUniform gebunden werden.

function CreateBumpMapSprite:TBumpMapSprite(colorMapGlId:int, alphaMapGlId:int, bumpMapGlId:int)
	local s:TBumpMapSprite = new TBumpMapSprite
	s.colorMapGlId = colorMapGlId
	s.alphaMapGlId = alphaMapGlId
	s.bumpMapGlId = bumpMapGlId
	return s
end function
'Laedt eine Textur und gibt die zugehoerige OpenGL-ID wieder.
function _LoadTexture:int(url:object)
	local pix:TPixmap = TPixmap(url)
	if not pix then
		pix = LoadPixmap(url)
		if not pix then return 0
	endif
	pix = pix.Convert(PF_RGBA8888)
	local glid:int
	glEnable(GL_TEXTURE_2D)
	glGenTextures(1, varptr glid)
	glBindTexture(GL_TEXTURE_2D, glid)
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, pix.width, pix.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, pix.pixels)
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	return glid
end function
'Laedt drei Texturen und erstellt eine BumpMap-Sprite daraus
function LoadBumpMapSprite:TBumpMapSprite(colorMapUrl:object, alphaMapUrl:object, bumpMapUrl:object)
	return CreateBumpMapSprite(_LoadTexture(colorMapUrl), _LoadTexture(alphaMapUrl), _LoadTexture(bumpMapUrl))
end function
type TBumpMapSprite

	field colorMapGlId:int, alphaMapGlId:int, bumpMapGlId:int

end type
'Stellt die Lichtrichtung ein. Der Vektor wird normiert. Wenigstens eine Komponente muss von 0 verschieden sein.
function SetLightDirection(x#, y#, z#)
	local v:TVector3 = CreateVector3(x, y, z)
	v.Normalize()
	glUniform3f(lightDirUniform.location, v.x, v.y, v.z)
	CheckError()
end function
'Uebergibt Color-, Alpha- und BumpMap an den Shader.
function SetBumpMapSprite(sprite:TBumpMapSprite)
	colorMapUniform.setTexture(sprite.colorMapGlId)
	alphaMapUniform.setTexture(sprite.alphaMapGlId)
	bumpMapUniform .setTexture(sprite. bumpMapGlId)
	CheckError
end function
'Sorgt dafuer, dass die Normalen der BumpMap um deg° gedreht werden.
function RotateNormals(deg#)
	glVertexAttrib2f(localXAttribute.location, cos(deg), sin(deg))
	glVertexAttrib2f(localYAttribute.location, -sin(deg), cos(deg))
end function
'Stellt den Vektor (x,y) als lokalen x-Vektor ein, d.h. diejenige Richtung, in welche die
'x-Komponente der BumpMap zu verstehen ist
'Darf fuer jeden Vertex einzeln aufgerufen werden.
'Der Vektor wird normiert. 
'Der lokale y-Vektor, d.h. diejenige Richtung, in welche die y-Komponente der BumpMap zu verstehen ist,
'wird hieraus berechnet.
function SetLocalX(x#, y#)
	local f# = 1.0/sqr(x*x+y*y)
	x :* f
	y :* f
	glVertexAttrib2f(localXAttribute.location, x, y)
	glVertexAttrib2f(localYAttribute.location, -y, x)
end function
