superstrict
Function CheckError() 'Prueft, ob irgendwelche OpenGL-Fehler vorliegen und wirft ggF. eine Exception
			' Beim Fehlersuchen kann man diese Funktion nicht oft genug aufrufen
        Local error:Int = glGetError()
        If error <> GL_NONE Then
        
                Local err$
                Select error
                        Case GL_INVALID_ENUM
                                err="GL_INVALID_ENUM"
                        Case GL_INVALID_VALUE
                                err="GL_INVALID_VALUE"
                        Case GL_INVALID_OPERATION
                                err="GL_INVALID_OPERATION"
                        Case GL_STACK_OVERFLOW
                                err="GL_STACK_OVERFLOW"
                        Case GL_STACK_UNDERFLOW
                                err="GL_STACK_UNDERFLOW"
                        Case GL_OUT_OF_MEMORY
                                err="GL_OUT_OF_MEMORY"
                        Default
                                err = "Unknown error"
                End Select
                Throw "GL Error: "+err
                
        EndIf
        
End Function
Function debugShader(shader:Int, name$ = "") 'Gibt das Info-Log des Shaders aus

	Local logLen:Int
	glGetShaderiv(shader, GL_INFO_LOG_LENGTH, Varptr logLen)
	If logLen Then
	
		Local buf:Byte Ptr = MemAlloc(logLen)
		Local anyIntThatIDonTNeedButOpenGL:Int
		glGetShaderInfoLog(shader, logLen, Varptr anyIntThatIDonTNeedButOpenGL, buf)
		Local str$ = String.FromBytes(buf, logLen)
		MemFree buf
		Print "DebugShader "+name+": "+str
		
	EndIf
	
	Print "DebugShader "+name+": "

End Function
Function debugProgram(prog:Int) ' Gibt das Info-Log des Programm-Objekts auf dem Bildschirm aus

	Local logLen:Int
	glGetProgramiv(prog, GL_INFO_LOG_LENGTH, Varptr logLen)
	If logLen Then
	
		Local buf:Byte Ptr = MemAlloc(logLen)
		Local anyIntThatIDonTNeedButOpenGL:Int
		glGetProgramInfoLog(prog, logLen, Varptr anyIntThatIDonTNeedButOpenGL, buf)
		Local str$ = String.FromBytes(buf, logLen)
		MemFree buf
		Print "DebugProgram: "+str
		
	EndIf
	
	Print "DebugProgram: "

End Function

type TOpenGLShader
rem
Zentrale Klasse zum Verwalten von Shadern.
Sie bietet Schnittstellen fuer folgende Funktionen:
 - Wechsel des aktuellen Shaders oder Wechsel in die feste Funktionspipeline
 - Zugriff auf Uniform- und Attributvariablen
 - Verwaltung von Texturen, die per Uniform uebergeben werden
endrem
	field programID:int=0, vertexShader:int=0, fragmentShader:int=0
	' Bei debug=True wird immer das Program-, Vertex- und Fragment-Log ausgegeben
	' Bei debug=False nur im Fehlerfall
	Global debug:int = true, activeShader:TOpenGLShader
	field texuniforms:TList = new TList
	field uniforms:TList = new TList
	field attributes:TList = new TList

	method Init:int(vertexUrl:object, fragmentUrl:object)
			'Laedt die Vertex- und Fragment-Shadersourcen
			'und erstellt das Shader-Objekt
		local vertexSource:Byte[] = LoadByteArray(vertexUrl)
		local fragmentSource:Byte[] = LoadByteArray(fragmentUrl)
		programID = glCreateProgramObjectARB()
		vertexShader=glCreateShaderObjectARB(GL_VERTEX_SHADER)
		fragmentShader=glCreateShaderObjectARB(GL_FRAGMENT_SHADER)
		local pointer:byte ptr, length:int
		pointer=varptr vertexSource[0]
		length=vertexSource.length
		glShaderSourceARB(vertexShader, 1, varptr pointer, varptr length)
		pointer=varptr fragmentSource[0]
		length=fragmentSource.length
		glShaderSourceARB(fragmentShader, 1, varptr pointer, varptr length)
		if debug then
			debugShader vertexShader, "VertexShader"
			debugShader fragmentShader, "FragmentShader"
		endif
		glCompileShader(vertexShader)
		glCompileShader(fragmentShader)
		if debug then
			debugShader vertexShader, "VertexShader"
			debugShader fragmentShader, "FragmentShader"
		endif
		local bool:int
		glGetShaderiv(vertexShader, GL_COMPILE_STATUS, Varptr bool)
		if not bool then
			debugShader vertexShader, "VertexShader"
			Destroy
			return False
		endif
		glGetShaderiv(fragmentShader, GL_COMPILE_STATUS, Varptr bool)
		if not bool then
			debugShader fragmentShader, "FragmentShader"
			Destroy
			return False
		endif
		glAttachShader(programID, vertexShader)
		glAttachShader(programID, fragmentShader)
		glLinkProgram(programID)
		if debug then debugProgram(programID)
		glDeleteShader(vertexShader)
		glDeleteShader(fragmentShader)
		glGetProgramiv(programID, GL_LINK_STATUS, Varptr bool)
		If Not bool Then
			if not debug then debugProgram(programID)
			Destroy
			return False
		endif
		return True
	end method
	method Destroy() 'Loescht den Shader
			 'Danach sollten keine weiteren Methoden mehr aufgerufen werden
			 'Diese Methode wird vom Destruktor aufgerufen

		if programID <> 0 then glDeleteObjectARB programID
		programID=0
	
	end method
	method Delete()
		Destroy
	end method
		'Die folgenden drei Funktionen geben Null zurueck, wenn kein Objekt zum Namen gefunden wurde:

	method GetUniform:TUniform(name$) 'Gibt das Uniform-Objekt zum Variablennamen zurueck
				          'Fuer Textur-Uniform-Variablen (Typ sampler2D) benutze
					  'man untenstehende Funktion
		local u:TUniform
		for u = eachin uniforms
			if u.name = name then return u
		next
		local location:int=glGetUniformLocation(programID, name)
		if location = -1 then return Null
		u = new TUniform
		u.name = name
		u.location = location
		uniforms.AddLast u
		return u
	end method
	method GetTextureUniform:TTextureUniform(name$) 'Gibt das Uniform-Objekt zum Variablennamen zurueck
							'Es sollte vom Typ sampler2D sein
							'Intern wird fuer dieses Uniform-Objekt
							'ein eigener Textur-Index reserviert.
							'Mit textureUniform.SetTexture(textureID) kann man
							'dann die Textur einstellen.
		local u:TTextureUniform
		for u = eachin texuniforms
			if u.name = name then return u
		next
		local location:int=glGetUniformLocation(programID, name)
		if location = -1 then return Null
		u = new TTextureUniform
		u.name = name
		u.location = location
		u.index=texuniforms.Count()
		texuniforms.AddLast u
		return u
	end method
	method GetAttribute:TVertexAttribute(name$) 'Gibt das Attribut-Objekt zum Variablennamen zurueck
		local a:TVertexAttribute
		for a = eachin attributes
			if a.name = name then return a
		next
		local location:int = glGetAttribLocation(programID, name)
		if location = -1 then return null
		a=new TVertexAttribute
		a.name=name
		a.location = location
		attributes.AddLast a
		return a
	end method
	function Change(changeto:TOpenGLShader, restoreTextures:int = True) 'Wechselt den aktiven Shader
				'Falls changeto=Null wird auf keinen Shader gewechselt
				'und es werden aktiv alle Texturen entbunden
				'Falls changeto <> Null und restoreTextures=True, werden die zuletzt
				'an die Uniform-Variablen gebundenen Texturen wiederhergestellt
		if activeShader = changeto then return
		if changeto = Null then
			for local i:int = 0 until activeShader.texuniforms.Count()
				glActiveTexture(GL_TEXTURE0+i)
				glBindTexture(GL_TEXTURE_2D, 0)
			next
			glActiveTexture(GL_TEXTURE0)
			glUseProgram 0
		else
			if activeShader <> Null then
				for local i:int = changeto.texuniforms.Count() until activeShader.texuniforms.Count()
					glActiveTexture(GL_TEXTURE0+i)
					glBindTexture(GL_TEXTURE_2D, 0)
				next
			endif
			glUseProgram changeto.programID
			if restoreTextures then changeto.RestoreTextures()
		endif
		activeShader = changeto
	end function
	method RestoreTextures()
		for local t:TTextureUniform = eachin texuniforms
			t.Restore
		next
	end method
end type
type TUniform
	field name$, location:int
end type
type TTextureUniform extends TUniform
	field index:int, value:int
	method SetTexture(tid:int)
		glActiveTexture(GL_TEXTURE0+index)
		glBindTexture(GL_TEXTURE_2D, tid)
		glUniform1i location, index
		value=tid
	end method
	method Restore()
		glActiveTexture(GL_TEXTURE0+index)
		glBindTexture(GL_TEXTURE_2D, value)
	end method
end type
type TVertexAttribute
	field name$, location:int
end type
