// Uniform-Variablen: Sind in Fragment- und Vertex-Shader verfuegbar.
// Sie sind fuer jedes gezeichnete Dreieck/Quad konstant.
// Texturen werden als Uniform uebergeben.
uniform sampler2D color;
uniform sampler2D alpha;
uniform sampler2D bump;
uniform vec3 lightDir;

// Attribute-Variablen: Sind nur im Vertex-Shader verfuegbar.
// Sie koennen fuer jeden einzelnen Vertex eingestellt werden
attribute vec3 normalVector; // Gewuenschter Normalenvektor
attribute vec2 localX;       // Lokale x-Richtung (d.h. die Richtung, in die die x-Werte der BumpMap zu verstehen sind)
attribute vec2 localY;       // Lokale y-Richtung

// Varying-Variablen: Sie werden vom Vertex-Shader gesetzt und vom Fragment-Shader ausgelesen.
// Fuer jedes Fragment (=Pixel) werden die Werte der drei zugehoerigen Vertices interpoliert.
// Daher koennen nur Float-aehnliche Werte (Floats, Vektoren, Matrizen) uebergeben werden.
varying vec2 uv;
varying vec3 normVec;
varying vec2 xDir;
varying vec2 yDir;

void main() {
	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
	uv = vec2(gl_MultiTexCoord0);
	normVec=normalVector;
	xDir = localX;
	yDir = localY;
}
